%{
    open Anormal
%}

%token <int> INT
%token <string> ID
%token TRUE
%token FALSE
%token PLUS
%token MINUS
%token TIMES
%token EQUALS
%token LT
%token GT
%token AND
%token OR
%token LPAREN
%token RPAREN
%token LET
%token LETREC
%token FN
%token IN
%token IF
%token THEN
%token ELSE
%token COMMA
%token ARROW
%token EOF

%nonassoc IN
%nonassoc ELSE
%left LT
%left GT
%left PLUS
%left TIMES
%left OR
%left AND

%start <Exp.t> prog
%%

prog:
| e = expr; EOF { e }
;

expr :
| i = INT { Const (I i) }
| x = ID { Var x }
| FN; LPAREN; args = separated_list(COMMA,ID); RPAREN; ARROW; body = expr { (Fun (args,body)) }
| TRUE { Const (B true) }
| FALSE { Const (B false) }
| e1 = expr; PLUS;  e2 = expr { PrimCall(Plus,[e1;e2]) }
| e1 = expr; MINUS; e2 = expr { PrimCall(Minus,[e1;e2]) }
| e1 = expr; TIMES; e2 = expr { PrimCall(Times,[e1;e2]) }
| e1 = expr; LT; e2 = expr { PrimCall(Lt,[e1;e2]) }
| e1 = expr; GT; e2 = expr { PrimCall(Gt,[e1;e2]) }
| e1 = expr; AND; e2 = expr { PrimCall(And,[e1;e2]) }
| e1 = expr; OR; e2 = expr { PrimCall(Or,[e1;e2]) }
| e1 = expr; EQUALS; e2 = expr { PrimCall(Equal,[e1;e2]) }
| LET; x = ID; EQUALS; e1 = expr; IN; e2 = expr { Bind(x,e1,e2) }
| LETREC; x = ID; LPAREN; args = separated_list(COMMA,ID); RPAREN; ARROW; body = expr; IN; e = expr { BindRec(x,args,body,e) }
| x = expr; LPAREN; args = separated_list(COMMA,expr); RPAREN { Call(x,args) }
| IF; e1 = expr; THEN; e2 = expr; ELSE; e3 = expr { If(e1,e2,e3) }
| LPAREN; e = expr; RPAREN {e}
;
